CON
  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

InputPin = 0
 Samples = 2000    '<- Number of Longs... Each Long contains 32 Bit Samples
                   'A value of 2000 captures 128,000 Bits ( = Samples * 64 )


FrequencyPin  = 0               '<- Used only for DEBUG 
Frequency = 20_000_000          '<- Used only for DEBUG
     

'Some Examples:
'Frequency =  2_500_000          'Pattern 33333333 33333333 or
                                 '        66666666 66666666 or
                                 '        CCCCCCCC CCCCCCCC or
                                 '        99999999 99999999
                                 
'Frequency =  5_000_000          'Pattern 55555555 55555555 or
                                 '        AAAAAAAA AAAAAAAA
                                 
'Frequency = 10_000_000          'Pattern 00000000 FFFFFFFF or
                                 '        FFFFFFFF 00000000
                                 
'Frequency = 20_000_000          'Pattern FFFFFFFF FFFFFFFF or
                                 '        00000000 00000000


  '512x192
  tiles = gr#tiles


OBJ
  gr    : "VGA.graphics"

OBJ
  Freq : "Synth"                '<- Used only for DEBUG  

VAR
    long Sync1
    long Sync2
    long StartBuffer1          
    long Buffer1[Samples]
    long Sync3
    long Sync4
    long StartBuffer2          
    long Buffer2[Samples]

PUB main|i


    Freq.Synth("A",FrequencyPin, Frequency)             'Start The Frequency Synthesizer '<- Used only for DEBUG

    FAST_DataCapture                                    'Capture the Data at 20MHz ( requires 4 COGS )

    gr.start
    gr.pointcolor(1)
    repeat i from 0 to tiles - 1                        'init tile colors to Scope-Green on Black
      gr.color(i,$BB00)

    gr.clear
    gr.text(55,0,string("FAST 20MHz BITstream Read"))   'Display results of the first 320 Bits

    gr.SimpleHEX(120,32,Buffer1[0],8)
    gr.SimpleHEX(120,64,Buffer1[1],8)
    gr.SimpleHEX(120,96,Buffer1[2],8)
    gr.SimpleHEX(120,128,Buffer1[3],8)            
    gr.SimpleHEX(120,160,Buffer1[4],8)

    gr.SimpleHEX(300,32,Buffer2[0],8)
    gr.SimpleHEX(300,64,Buffer2[1],8)
    gr.SimpleHEX(300,96,Buffer2[2],8)
    gr.SimpleHEX(300,128,Buffer2[3],8)            
    gr.SimpleHEX(300,160,Buffer2[4],8)

    repeat                                              'Keep COG alive



PUB FAST_DataCapture
    repeat until Sync1*Sync2*Sync3*Sync4 <> 0           'Chances are slim (about 4 in 2^32) but just do this anyway.
                                                        '  basically we don't want Sync1,Sync2,Sync3, or Sync4 to equal Zero
                                                        '  If one of them happens to be Zero, it could cause the Assembly
                                                        '  program to improperly align to the DataBuffer.
                                                           
      Sync1 := 50_000 + cnt                             'project a future cnt event for Sync1
                                                        'Note: if cnt rolls over, this is ok... it will rollover in the offset as well maintaining
                                                        'an absolute reference.

      Sync2 := Sync1 + 256                              'Adjust Sync2 so that it is a product of Sync1 plus 256 Clocks

      Sync3 := Sync1 + 4                                'Adjust Sync3 so that it is a product of Sync1 plus 4 Clocks
      Sync4 := Sync2 + 4                                'Adjust Sync4 so that it is a product of Sync2 plus 4 Clocks

    StartBuffer1 := 0                                   'Always Zero used as a Flag for Buffer1 memory location
    cognew(@FastDataIN,@Sync1)                          'Start a new instance of FastDataIN
    cognew(@FastDataIN,@Sync2)                          'Start a new instance of FastDataIN

    StartBuffer2 := 0                                   'Always Zero used as a Flag for Buffer2 memory location
    cognew(@FastDataIN,@Sync3)                          'Start a new instance of FastDataIN
    cognew(@FastDataIN,@Sync4)                          'Start a new instance of FastDataIN
       
    waitcnt(Sync4)                                      'Wait Here until Data is ready
 
DAT
'############################################################################################
'############################################################################################
FastDataIN    org       0
              mov       Sync,                   par
              rdlong    Sync,                   Sync
              mov       InputPinMask,           #1
              shl       InputPinMask,           #InputPin
              mov       LoopCounter,            LoopCounts
              mov       BufferOffset,           #0
              mov       BufferIndex,            par
FindBuffer    add       BufferIndex,            #8
              add       BufferOffset,           #4
              rdlong    BufferVal,              BufferIndex
              tjnz      BufferVal,              #FindBuffer               
              mov       BufferLocation,         BufferIndex
              add       BufferLocation,         BufferOffset

Loop1         waitcnt   Sync, TTLC

              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff       
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff       
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff       
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff       
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff
              test      InputPinMask, ina    wc        '(4-clocks)Check I/O pin for a "1" or a "0" ; set "C" flag
              rcl       Buff,#1                        '(4-clocks)Rotate the "C" flag LEFT into Buff

'                                                       256-Clocks Total

              wrlong    Buff,                   BufferIndex
              add       BufferIndex,            #8
              djnz      LoopCounter,            #Loop1

              cogid     temp                    
              cogstop   temp                    'STOP this COG

              
'Initialized variables
TTLC                    long    512
LoopCounts              long    Samples / 2

'Uninitialized variables
LoopCounter             long    0
Buff                    long    0
InputPinMask            long    0
Sync                    long    0
BufferIndex             long    0
BufferOffset            long    0
BufferLocation          long    0

'Aliased variables
temp                    long    0
BufferVal
 